/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */
package com.microej.example.bluetooth.hello;

import java.util.logging.Logger;

import ej.bluetooth.util.server.BasicBluetoothServer;
import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.components.dependencyinjection.ServiceLoaderFactory;
import ej.wadapps.app.BackgroundService;

/**
 * A background service mounting a Bluetooth server and sending an Hello World every 5s.
 */
public class HelloWorldBackgroundService extends TimerTask implements BackgroundService {

	/**
	 * Logger for the application.
	 */
	public static final Logger LOGGER = Logger.getLogger("helloWorld"); //$NON-NLS-1$
	private static final String DEVICE_NAME = "MicroEJ Device"; //$NON-NLS-1$
	private static final long RATE = 5_000;
	private BasicBluetoothServer basicBluetoothServer;
	private HelloWorldService helloWorldService;

	@Override
	public void onStart() {
		this.basicBluetoothServer = new BasicBluetoothServer(DEVICE_NAME);
		this.helloWorldService = new HelloWorldService();
		this.basicBluetoothServer.addService(this.helloWorldService);
		LOGGER.info("Starting bluetooth server"); //$NON-NLS-1$
		this.basicBluetoothServer.start();
		ServiceLoaderFactory.getServiceLoader().getService(Timer.class, Timer.class).schedule(this, RATE, RATE);
	}

	@Override
	public void onStop() {
		LOGGER.info("Stopping bluetooth server"); //$NON-NLS-1$
		this.basicBluetoothServer.stop();
		cancel();
	}

	@Override
	public void run() {
		LOGGER.info("Request notification"); //$NON-NLS-1$
		this.basicBluetoothServer.notifyDevices(this.helloWorldService.getHelloWorldCharacteristic());
	}

}
